const dirName = __dirname;

const electron = require('electron');
const { app, dialog, screen, shell, ipcMain, BrowserWindow, nativeImage, powerMonitor, powerSaveBlocker, Tray, crashReporter } = electron;
const path = require('path');
const fs = require('fs');
const os = require('os');
const portfinder = require('portfinder');
const Server = require(dirName + '/server.js');

require('@electron/remote/main').initialize();

const appName = app.getName();
const appVersion = app.getVersion();
const userData = app.getPath('userData');



let shareData = {
					'appName': appName,
					'appVersion': appVersion,
					'apiKey': '',
					'apiUrl': '',
					'appIcon': '',
					'dataStore': '',
					'userData': userData
				};


portfinder.basePort = 9000;
portfinder.highestPort = 60000;

let serverPort = 0;

const apiTimeoutMins = 10;

const appExpress = Server.app;

crashReporterStart();

//app.disableHardwareAcceleration();

// Ensure only one instance is running
if (app.requestSingleInstanceLock() == false) {

	app.quit();
}


portfinder.getPort((err, port) => {

	serverPort = port;
	//console.log(port);
});




// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
// Some APIs can only be used after this event occurs.
app.whenReady().then(() => {

	switch (process.platform) {

		case 'win32': shareData['appIcon'] = path.resolve(dirName, 'icons/icon_windows.ico'); break;
		case 'darwin': shareData['appIcon'] = path.resolve(dirName, 'icons/icon_mac.icns'); break;
		case 'linux': shareData['appIcon'] = path.resolve(dirName, 'icons/icon_256x256.png'); break;
	}

	let dir = userData + '/data';
	let dirUpload = userData + '/data/uploads';

	let osType = os.type();
	let osRelease = os.release();
	let osArch = os.arch();
	let osPlatform = os.platform();

	if (process.platform == 'win32') {

		app.setUserTasks([{
							program: process.execPath,
							arguments: '--new-window',
							iconPath: shareData['appIcon'],
							iconIndex: 0,
							title: appName,
							description: appName
						}]);
	}

	if (!fs.existsSync(dir)) {

		fs.mkdirSync(dir);
	}

	if (!fs.existsSync(dirUpload)) {

		fs.mkdirSync(dirUpload);
	}

	try {

		fs.unlinkSync(dir + '/app.dat');
		fs.unlinkSync(dir + '/news.dat');
		fs.unlinkSync(dir + '/markets.dat');
		fs.unlinkSync(dir + '/signals.dat');
		fs.unlinkSync(dir + '/symbols.dat');
	}
	catch(err) {

		//console.error(err);
	}

	// Prevent sleep
	powerMonitor.on("lock-screen", () => {

		let id = powerSaveBlocker.start("prevent-display-sleep");
	});

	powerMonitor.on("suspend", () => {

		powerSaveBlocker.start("prevent-app-suspension");
	});

	//app.commandLine.appendSwitch('high-dpi-support', 1)
	//app.commandLine.appendSwitch('force-device-scale-factor', 1)
	app.commandLine.appendSwitch('js-flags', '--max-old-space-size=16384');

	let server = appExpress.listen(serverPort, '0.0.0.0', function () {

		const host = server.address().address;
		const port = server.address().port;

		const apiTimeout = (60 * 1000) * apiTimeoutMins;

		const keepAliveTimeout = 1000 * 120;
		const headersTimeout = keepAliveTimeout + (1000 * 5);

		// Setting to zero may resolve ECONNRESET
		//server.setTimeout(0);

		server.setTimeout(apiTimeout);

		server.keepAliveTimeout = keepAliveTimeout;
		server.headersTimeout = headersTimeout;

		console.log(appName + " " + appVersion  + " ready at http://%s:%s", host, port, "\n");

	});

	const mainWindow = createMainWindow();

	shareData['shell'] = shell;
	shareData['electron'] = app;
	shareData['mainWindow'] = mainWindow;
	shareData['BrowserWindow'] = BrowserWindow;
	shareData['userAgent'] = '3CQS/' + shareData.appVersion + ' (' + osPlatform + ' ' + osArch + '; ' + osType + ' ' + osRelease + ')';

	Server.init(shareData);
	Server.loadConfig();
	//Server.checkStatus();

	app.on('activate', function () {

		// On macOS it's common to re-create a window in the app when the
		// dock icon is clicked and there are no other windows open.

		if (BrowserWindow.getAllWindows().length === 0) { createMainWindow(); }
	})
});


process.on('uncaughtException', function (error) {

	showError(error);
});


app.on('render-process-gone', (e, webContents, details) => {

	showError(details);
});


app.on('gpu-process-crashed', (event, killed) => {

	let obj = { 'event': event, 'killed': killed };

	showError(obj);
});


// Quit when all windows are closed, except on macOS. There, it's common
// for applications and their menu bar to stay active until the user quits
// explicitly with Cmd + Q.
app.on('window-all-closed', function () {

	if (process.platform !== 'darwin') { app.quit(); }
});


app.on('before-quit', function () {


});


ipcMain.on('app-quit', (evt, arg) => {

	app.quit();
})


ipcMain.on('data-store', (e, data, web_component_id) => {

	shareData['dataStore'] = data;
	//console.log(shareData['dataStore']);

    //send data back
    //ipcMain.sendTo(web_component_id, 'data-store-reply');
});


ipcMain.on('toMain', (event, args) => {

	let data = {
					'appIcon': shareData['appIcon'],
					'dataKey': shareData['dataKey']
			   };

	Server.dataSave('app.dat', data);

	event.sender.send('fromMain', data);
});


ipcMain.on('mainLogData', (event, hash, args) => {

	Server.logData(hash, args);
});


ipcMain.on('mainCreateBrowser', (event, hash, args) => {

	Server.createBrowser(hash);
});


ipcMain.handle('get-metrics', async (event, data) => {

	let metrics = app.getAppMetrics();

	return metrics;
});





function showError(error) {

	let metrics = app.getAppMetrics();

	dialog.showErrorBox('Oops! Something went wrong!', JSON.stringify(error));

	app.relaunch();
	app.exit();
}


function crashReporterStart() {

	const crashReporterOptions = {

		companyName: '3CQS',
		productName: '3CQS',
		submitURL: '',
		autoSubmit: false,
		uploadToServer: false,
		ignoreSystemCrashHandler: false,
		compress: true,
	};

	crashReporter.start(crashReporterOptions);
}


function createMainWindow () {

	const showDevTools = false;

	let trayIcon = nativeImage.createFromPath(dirName + '/icons/icon_256x256.png');
	trayIcon = trayIcon.resize({ width: 16, height: 16 });

	const { width, height } = electron.screen.getPrimaryDisplay().workAreaSize;
	//let win = new BrowserWindow({width, height});

	// Create the browser window.
	const mainWindow = new BrowserWindow({

		show: false,
		width: width - 30,
		height: height,
        icon: shareData['appIcon'],
		webPreferences: {

			preload: path.join(dirName, 'preload.js'),
			//webSecurity: false,
			//allowRunningInsecureContent: true,
			//contextIsolation: false,
			backgroundThrottling: false,
			enableRemoteModule: true,
			nativeWindowOpen: true,
			nodeIntegration: true,
			devTools: showDevTools
		}
	});


	mainWindow.once('ready-to-show', () => {

		//mainWindow.show();
	});


	mainWindow.on('resize', function () {

		let size = mainWindow.getSize();

		let width = size[0];
		let height = size[1];

		let obj = { 'width': width, 'height': height };

		mainWindow.webContents.send('resized', obj);
	});

	let wc = mainWindow.webContents;

	require('@electron/remote/main').enable(wc);

	wc.on('did-finish-load', () => {

		Server.setWindowDefaults();
	});


	// Force all hyperlinks to open in new browser window
	wc.on('will-navigate', function (e, url) {

		let urlParts = new URL(url);

		if ((urlParts['hostname'] == '127.0.0.1' || urlParts['hostname'] == 'localhost') && urlParts['pathname'] == '/exit') {

			// Don't open browser window on exit
		}
		else if (url != wc.getURL()) {

			e.preventDefault();
			shell.openExternal(url);
		}
	});

	//mainWindow.setOverlayIcon(trayIcon, appName);

	shareData['remoteClientKey'] = Server.genUUID();

    const appIconTray = new Tray(trayIcon);
	const options = { extraHeaders: 'X-API-Key: ' + shareData['remoteClientKey'] };

	let offsetHeight = 30;

	let winSize = mainWindow.getSize();

	mainWindow.setSize(winSize[0], parseInt(winSize[0] * 9 / 16) - offsetHeight);

	// and load the index.html of the app.
	//  mainWindow.loadFile('public/index.html')

	mainWindow.loadURL('http://127.0.0.1:' + serverPort + '/main', options);

	mainWindow.focus();

	if (showDevTools) {

		mainWindow.webContents.openDevTools();
	}

	return mainWindow;
}

